@@ -8,10 +8,11 @@
 #
 ######################################################################
 
-import re
 import argparse
 import os
+import re
 import templar.link as link
+from templar import log
 from templar.markdown import Markdown
 
 
@@ -32,6 +33,8 @@ def process_inheritance(template_path, template_dirs):
 def process(template, attrs):
     if not isinstance(attrs, Frame):
         attrs = Frame(bindings=attrs)
+    template, macros = process_def(template)
+    template = process_call(template, attrs, macros)
     template = process_control(template, attrs)
     while expr_re.search(template):
         template = expr_re.sub(lambda m: str(evaluate(m.group(1),
@@ -94,6 +97,42 @@ for_re = r"""
     %\}
 """
 
+def_re = re.compile(r"""
+    \{%                     # opening brace
+        \s*def\s+
+        (\w+)               # \1 is the macro name
+        \s*
+        (                   # \2 is variable list
+            (?:
+                \{{2}\s*    # variable is of the form {{ var }}
+                .*?         # variable name
+                \s*\}{2}
+                \s*,?\s*    # comma
+            )+              # one or more variables
+        )
+    \s*%\}
+    (.*?)                   # \3 is macro body
+    \{%                     # closing brace
+        \s*enddef\s*
+    %\}
+""", re.S | re.X)
+
+call_re = re.compile(r"""
+    \{%
+        \s*call\s+
+        (\w+)               # \1 is macro name
+        \s*
+        (                   # \2 is variable list
+            (?:
+                \{{2}\s*    # argument is of the form {{ var }}
+                .*?         # argument
+                \s*\}{2}
+                \s*,?\s*    # comma
+            )+              # one or more arguments
+        )
+    \s*%\}
+""", re.S | re.X)
+
 def conditional_re(keyword):
     return r"""
     \{%                 # brace
@@ -148,6 +187,34 @@ def process_if(match, attrs):
             return process(body, attrs).strip()
     return ''
 
+def process_call(template, attrs, macros):
+    def call_sub(match):
+        macro, arguments = match.groups()
+        arguments = expr_re.findall(arguments)
+        if macro not in macros:
+            log.warn('The macro "' + macro + '" is not defined.')
+            return ''
+        variables, body = macros[macro]
+        new_frame = Frame(parent=attrs)
+        for var, arg in zip(variables, arguments):
+            new_frame[var] = evaluate(arg, attrs)
+        return expr_re.sub(lambda m: eval_macro_body(m, new_frame), body.strip('\n'))
+    def eval_macro_body(match, attrs):
+        result = evaluate(match.group(1), attrs)
+        if not result:
+            return match.group(0)
+        return str(result)
+    return call_re.sub(call_sub, template)
+
+def process_def(template):
+    macros = {}
+    for macro, variables, body in def_re.findall(template):
+        variables = expr_re.findall(variables.replace('\n', ' '))
+        macros[macro] = (variables, body)
+    template = def_re.sub('', template)
+    return template, macros
+
+
 def is_true(expression, attrs):
     if evaluate(expression, attrs):
         return True
@@ -229,9 +296,10 @@ def get_template(filename, template_dirs):
         template = os.path.join(path, 'templates', filename)
         if file_exists(template):
             return file_read(template)
-    print(filename + ' could not be found in:')
+    log.warn('The template "' + filename \
+            + '" could not be found in these directories:')
     for path in dirs:
-        print(os.path.join(path, 'templates'))
+        log.log(os.path.join(path, 'templates'))
     exit(1)
 
 
@@ -380,6 +448,9 @@ def file_write(filename, text):
     NOTE:
     This function exists to make test injection easier.
     """
+    dirname = os.path.dirname(filename)
+    if not os.path.exists(dirname):
+        os.makedirs(dirname)
     with open(filename, 'w') as f:
         f.write(text)
 
@@ -401,35 +472,34 @@ def cmd_options(parser):
 def main(args, configs):
     if args.source:
         if not file_exists(args.source):
-            print('File ' + args.source + ' does not exist.')
+            log.warn('File ' + args.source + ' does not exist.')
             exit(1)
         elif not os.path.isfile(args.source):
-            print(args.source + ' is not a valid file')
+            log.warn(args.source + ' is not a valid file')
             exit(1)
-        result = link.link(file_read(args.source))
+        result = link.link(args.source)
         if args.markdown:
             markdown_obj = Markdown(result)
             for k, v in markdown_obj.variables.items():
-                configs[k] = v
+                configs['VARIABLES'][k] = v
             result = markdown_obj.text
         result = link.substitutions(result,
                                     configs.get('SUBSTITUTIONS', []),
                                     args)
         result, cache = link.retrieve_blocks(result)
         if 'TOC_BUILDER' in configs:
-            configs['table-of-contents'] = link.scrape_headers(result, configs['TOC_BUILDER'])
+            configs['VARIABLES']['table-of-contents'] = link.scrape_headers(result, configs['TOC_BUILDER'])
             for import_stmt in (
                     'from datetime import datetime',
                     ):
                 exec(import_stmt, configs)
         for k, v in cache.items():
-            configs[k] = v
+            configs['VARIABLES'][k] = v
 
     result = compile(args.template, configs)
     if not args.destination:
-        print(result)
+        log.log(result)
         return
     file_write(args.destination, result)
-    print('Finished compiling')
-    print('Result can be found at ' + args.destination)
+    log.info('Result can be found at ' + args.destination)
 
